@@ -52,6 +52,8 @@
 #include "shl_misc.h"
 #include "text.h"
 #include "uterm_video.h"
+#include "text_gltex_atlas.frag.bin.h"
+#include "text_gltex_atlas.vert.bin.h"
 
 #define LOG_SUBSYSTEM "text_gltex"
 
@@ -141,11 +143,6 @@ static void free_glyph(void *data)
 	free(glyph);
 }
 
-extern const char _binary_src_text_gltex_atlas_vert_bin_start[];
-extern const char _binary_src_text_gltex_atlas_vert_bin_end[];
-extern const char _binary_src_text_gltex_atlas_frag_bin_start[];
-extern const char _binary_src_text_gltex_atlas_frag_bin_end[];
-
 static int gltex_set(struct kmscon_text *txt)
 {
 	struct gltex *gt = txt->data;
@@ -162,14 +159,12 @@ static int gltex_set(struct kmscon_text *txt)
 	shl_dlist_init(&gt->atlases);
 
 	ret = shl_hashtable_new(&gt->glyphs, shl_direct_hash,
-				shl_direct_equal, NULL,
-				free_glyph);
+				shl_direct_equal, free_glyph);
 	if (ret)
 		return ret;
 
 	ret = shl_hashtable_new(&gt->bold_glyphs, shl_direct_hash,
-				shl_direct_equal, NULL,
-				free_glyph);
+				shl_direct_equal, free_glyph);
 	if (ret)
 		goto err_htable;
 
@@ -180,10 +175,10 @@ static int gltex_set(struct kmscon_text *txt)
 		goto err_bold_htable;
 	}
 
-	vert = _binary_src_text_gltex_atlas_vert_bin_start;
-	vlen = _binary_src_text_gltex_atlas_vert_bin_end - vert;
-	frag = _binary_src_text_gltex_atlas_frag_bin_start;
-	flen = _binary_src_text_gltex_atlas_frag_bin_end - frag;
+	vert = _binary_text_gltex_atlas_vert_start;
+	vlen = _binary_text_gltex_atlas_vert_size;
+	frag = _binary_text_gltex_atlas_frag_start;
+	flen = _binary_text_gltex_atlas_frag_size;
 	gl_clear_error();
 
 	ret = gl_shader_new(&gt->shader, vert, vlen, frag, flen, attr, 4,
@@ -411,8 +406,7 @@ static int find_glyph(struct kmscon_text *txt, struct glyph **out,
 	else
 		font->attr.italic = false;
 
-	res = shl_hashtable_find(gtable, (void**)&glyph,
-				 (void*)(uint64_t)id);
+	res = shl_hashtable_find(gtable, (void**)&glyph, id);
 	if (res) {
 		*out = glyph;
 		return 0;
@@ -515,7 +509,7 @@ static int find_glyph(struct kmscon_text *txt, struct glyph **out,
 	glyph->atlas = atlas;
 	glyph->texoff = atlas->fill;
 
-	ret = shl_hashtable_insert(gtable, (void*)(uint64_t)id, glyph);
+	ret = shl_hashtable_insert(gtable, id, glyph);
 	if (ret)
 		goto err_free;
 
